import os
import re
import osgtest.library.core as core
import osgtest.library.files as files
import osgtest.library.osgunittest as osgunittest

class TestCondorCE(osgunittest.OSGTestCase):

    required_rpms = ['htcondor-ce',
                     'htcondor-ce-client',
                     'lcmaps',
                     'lcas-lcmaps-gt4-interface']

    required_condor_rpms = ['condor', 'htcondor-ce-condor']
    
    required_pbs_rpms = ['torque-mom',
                         'torque-server', 
                         'torque-scheduler',
                         'torque-client', # for qmgr
                         'munge',
                         'htcondor-ce-pbs']

    def test_01_status(self):
        core.skip_ok_unless_installed(*self.required_rpms)
        core.skip_ok_unless_installed(*self.required_condor_rpms)
        
        command = ('condor_ce_status', '-verbose')
        core.check_system(command, 'ce status', user=True)

    def test_02_queue(self):
        core.skip_ok_unless_installed(*self.required_rpms)
        core.skip_ok_unless_installed(*self.required_condor_rpms)
        
        command = ('condor_ce_q', '-verbose')
        core.check_system(command, 'ce queue', user=True)
        
    def test_03_ping_with_gridmap_auth(self):
        core.skip_ok_unless_installed(*self.required_rpms)
        core.skip_ok_unless_installed(*self.required_condor_rpms)

        command = ('env', '_condor_SEC_CLIENT_AUTHENTICATION_METHODS=GSI', 'condor_ce_ping', 'WRITE', '-verbose')
        stdout, _, _ = core.check_system(command, 'ping using gridmap', user=True)
        self.assert_(re.search('Authorized:\s*TRUE', stdout), 'could not authorize with gridmap')

    def test_04_condor_trace(self):
        core.skip_ok_unless_installed(*self.required_rpms)
        core.skip_ok_unless_installed(*self.required_condor_rpms)

        cwd = os.getcwd()
        os.chdir('/tmp')
        
        command = ('condor_ce_trace', '--debug', core.get_hostname())
        core.check_system(command, 'condor ce trace', user=True)

        os.chdir(cwd)

    def test_05_pbs_trace(self):
        core.skip_ok_unless_installed(*self.required_rpms)
        core.skip_ok_unless_installed(*self.required_pbs_rpms)
        
        cwd = os.getcwd()
        os.chdir('/tmp')
        
        command = ('condor_ce_trace', '--debug', core.get_hostname())
        core.check_system(command, 'pbs ce trace', user=True)

        os.chdir(cwd)

    # Reconfigure condorce to use GUMS for authorization
    def test_06_stop_condorce(self):
        core.skip_ok_unless_installed(*self.required_rpms)
        core.skip_ok_unless_installed(*self.required_condor_rpms)
        core.skip_ok_unless_installed('gums-service')
        self.skip_bad_if(core.state['condor-ce.started'] == False, 'condor-ce already down')

        command = ('service', 'condor-ce', 'stop')
        stdout, _, fail = core.check_system(command, 'Stop HTCondor CE')
        self.assert_(stdout.find('error') == -1, fail)
        self.assert_(not os.path.exists(core.config['condor-ce.lockfile']),
                     'Condor run lock file still present')

        core.state['condor-ce.started'] = False

    def test_07_configure_gums(self):
        core.skip_ok_unless_installed(*self.required_rpms)
        core.skip_ok_unless_installed(*self.required_condor_rpms)
        core.skip_ok_unless_installed('gums-service')

        hostname = core.get_hostname()
        
        core.config['condor-ce.gsi-authz'] = '/etc/grid-security/gsi-authz.conf'
        gsiauthz_contents = "globus_mapping liblcas_lcmaps_gt4_mapping.so lcmaps_callout"
        files.write(core.config['condor-ce.gsi-authz'], gsiauthz_contents, owner='condor-ce')
        
        core.config['condor-ce.gums-client'] = '/etc/gums/gums-client.properties'
        gumsclient_contents = """gums.location=https://%s:8443/gums/services/GUMSAdmin
gums.authz=https://%s:8443/gums/services/GUMSXACMLAuthorizationServicePort
""" % (hostname, hostname)
        files.write(core.config['condor-ce.gums-client'], gumsclient_contents, owner='condor-ce')        

        core.config['condor-ce.lcmapsdb'] = '/etc/lcmaps.db'
        lcmaps_contents = """gumsclient = "lcmaps_gums_client.mod"
             "-resourcetype ce"
             "-actiontype execute-now"
             "-capath /etc/grid-security/certificates"
             "-cert   /etc/grid-security/hostcert.pem"
             "-key    /etc/grid-security/hostkey.pem"
             "--cert-owner root"
# Change this URL to your GUMS server
             "--endpoint https://%s:8443/gums/services/GUMSXACMLAuthorizationServicePort"
# Uncomment this to set a different expected host certificate name for server
#            "--override-expected-hostname overridegumsname.yourdomain"

# lcmaps policies require at least two modules, so these are here to
#   fill in if only one module is needed.  "good | bad" has no effect.
good        = "lcmaps_dummy_good.mod"
bad         = "lcmaps_dummy_bad.mod"

authorize_only:
gridmapfile -> good | bad
""" % hostname

        files.write(core.config['condor-ce.lcmapsdb'], lcmaps_contents, backup=False)

    def test_08_start_condorce(self):
        core.skip_ok_unless_installed(*self.required_rpms)
        core.skip_ok_unless_installed(*self.required_condor_rpms)
        core.skip_ok_unless_installed('gums-service')

        self.skip_ok_if(os.path.exists(core.config['condor-ce.lockfile']), 'already running')

        command = ('service', 'condor-ce', 'start')
        stdout, _, fail = core.check_system(command, 'Start HTCondor CE')
        self.assert_(stdout.find('FAILED') == -1, fail)
        self.assert_(os.path.exists(core.config['condor-ce.lockfile']),
                     'HTCondor CE run lock file missing')
        core.state['condor-ce.started'] = True

    def test_09_ping_with_gums_auth(self):
        core.skip_ok_unless_installed(*self.required_rpms)
        core.skip_ok_unless_installed(*self.required_condor_rpms)
        core.skip_ok_unless_installed('gums-service')

        command = ('env', '_condor_SEC_CLIENT_AUTHENTICATION_METHODS=GSI', 'condor_ce_ping', 'WRITE', '-verbose')
        stdout, _, _ = core.check_system(command, 'ping using GUMS', user=True)
        self.assert_(re.search('Authorized:\s*TRUE', stdout), 'could not authorize with GUMS')
